#ifndef __USER_ADAU1761__
#define __USER_ADAU1761__
/*==============================================================================
    FILE:           user_adau1979_1962a.h

    DESCRIPTION:    Provides a interface between the user_audio USB Audio 2.0
                    functionality and the ADI adau1979 and asau1962a drivers.

    Copyright (c) 2018 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to use, reproduce and distribute this source
    file as well as executable files created using this source file for use with 
    Analog Devices SC5xx family processors only. Nothing else gives you 
    the right to use this source file.

==============================================================================*/
/*!
 * @file      user_adau1979_1962a.h
 * @brief     Interface logic for the ADAU1979 and ADAU1962a drivers.
 *
 * @details
 *            Provides an interface between the user_audio USB Audio 2.0
 *            functionality and the ADI adau1979 and adau1962a drivers.
 *
 *            Copyright (c) 2018 Closed Loop Design, LLC
 *
 *            This software is supplied "AS IS" without any warranties, express, implied
 *            or statutory, including but not limited to the implied warranties of fitness
 *            for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
 *            extends you a royalty-free right to use, reproduce and distribute this source
 *            file as well as executable files created using this source file for use with 
 *            Analog Devices SC5xx family processors only. Nothing else gives you 
 *            the right to use this source file.

 *
 */


#include "codec/adi_adau1979.h"
#include "codec/adi_adau1962a.h"

#define NUM_AUDIO_CHANNELS                      8

/**
 * Return codes for the user_adau1979_1962a_init function.
 */
typedef enum
{
    USER_ADAU1979_1962a_INIT_SUCCESS = 0,   /*!< Initialization successful */
    USER_ADAU1979_1962a_INIT_ONGOING,       /*!< Initialization is ongoing */
    USER_ADAU1979_1962a_INIT_FAILED,        /*!< Initialization failed */
} User_ADAU1979_1962a_Init_Return_Code;

/**
 *Structure used to send USB audio data to the ADAU1962A Codec
 */
typedef struct
{
    unsigned short buffer_size;             /*!< Number of bytes being transferred */
    unsigned char * p_buffer;               /*!< Pointer to the data being transfered */
    void (*fp_xfr_complete) (void);         /*!< Function called once the data is sent
                                               to the codec. */
} User_ADAU1979_1962a_Audio_Transfer_Params;

extern User_ADAU1979_1962a_Init_Return_Code user_adau1979_1962a_init (void);
extern void user_adau1979_1962a_main (void);
extern void user_adau1962a_set_volume_headphones (
    ADI_ADAU1962A_CHNL                   eChannel,
    unsigned short                 const usb_volume);
extern void user_adau1979_set_volume_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel,
    unsigned short                 const usb_volume);

extern void user_adau1962a_set_mute_headphones (
    ADI_ADAU1962A_CHNL                   eChannel,
    unsigned char                         mute);

extern void user_adau1979_set_mute_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel,
    unsigned char                         mute);

extern void user_adau1962a_get_volume_headphones (
    ADI_ADAU1962A_CHNL                   eChannel,
    unsigned short *                p_volume);

extern void user_adau1979_get_volume_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel,
    unsigned short *                p_volume);

extern unsigned char user_adau1962a_get_mute (
    ADI_ADAU1962A_CHNL                   eChannel);

extern unsigned char user_adau1979_get_mute_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel);

extern void user_adau1962a_disable_output (void);
extern void user_adau1962a_enable_output (void);
extern void user_adau1979_enable_input (void);
extern void user_adau1979_disable_input (void);
extern float user_adau1962a_get_feedback_value (void);
#endif /* __USER_ADAU1761__ */
